@@ -22,43 +22,11 @@
 #include <ags/X/ags_machine.h>
 #include <ags/X/ags_machine_callbacks.h>
 
-#include <ags/object/ags_application_context.h>
-#include <ags/object/ags_soundcard.h>
-
-#include <ags/widget/ags_led.h>
-
-#include <ags/thread/ags_mutex_manager.h>
-
-#include <ags/audio/ags_audio.h>
-#include <ags/audio/ags_input.h>
-#include <ags/audio/ags_output.h>
-#include <ags/audio/ags_audio_signal.h>
-#include <ags/audio/ags_playback_domain.h>
-#include <ags/audio/ags_playback.h>
-#include <ags/audio/ags_pattern.h>
-#include <ags/audio/ags_recall.h>
-#include <ags/audio/ags_recall_container.h>
-
-#include <ags/audio/thread/ags_audio_loop.h>
-
-#include <ags/audio/task/ags_toggle_pattern_bit.h>
-
-#include <ags/audio/task/recall/ags_apply_bpm.h>
-#include <ags/audio/task/recall/ags_apply_sequencer_length.h>
-
-#include <ags/audio/recall/ags_delay_audio.h>
-#include <ags/audio/recall/ags_count_beats_audio.h>
-#include <ags/audio/recall/ags_count_beats_audio_run.h>
-#include <ags/audio/recall/ags_copy_pattern_audio.h>
-#include <ags/audio/recall/ags_copy_pattern_audio_run.h>
-#include <ags/audio/recall/ags_copy_pattern_channel.h>
-#include <ags/audio/recall/ags_copy_pattern_channel_run.h>
-#include <ags/audio/recall/ags_play_channel.h>
-#include <ags/audio/recall/ags_play_channel_run.h>
-#include <ags/audio/recall/ags_play_audio_signal.h>
-
-#include <ags/audio/file/ags_audio_file.h>
+#include <ags/libags.h>
+#include <ags/libags-audio.h>
+#include <ags/libags-gui.h>
 
+#include <ags/X/ags_ui_provider.h>
 #include <ags/X/ags_window.h>
 #include <ags/X/ags_pad.h>
 #include <ags/X/ags_navigation.h>
@@ -166,97 +134,105 @@ ags_drum_open_response_callback(GtkDialog *dialog, gint response, AgsDrum *drum)
 void
 ags_drum_loop_button_callback(GtkWidget *button, AgsDrum *drum)
 {
+  AgsPort *port;
+  
   AgsCountBeatsAudio *count_beats_audio;
 
-  AgsMutexManager *mutex_manager;
-
-  GList *list;
+  GList *start_list, *list;
 
   gboolean loop;
 
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-  
-  pthread_mutex_lock(application_mutex);
-  
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) AGS_MACHINE(drum)->audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
-  pthread_mutex_lock(audio_mutex);
-
   loop = (GTK_TOGGLE_BUTTON(button)->active) ? TRUE: FALSE;
 
-  /* AgsCopyPatternAudio */
-  list = AGS_MACHINE(drum)->audio->play;
+  /* play - count beats audio */
+  g_object_get(AGS_MACHINE(drum)->audio,
+	       "play", &start_list,
+	       NULL);
+  
+  list = start_list;
 
   while((list = ags_recall_find_type(list,
 				     AGS_TYPE_COUNT_BEATS_AUDIO)) != NULL){
+    GValue value = {0,};
+    
     count_beats_audio = AGS_COUNT_BEATS_AUDIO(list->data);
+    g_object_get(count_beats_audio,
+		 "sequencer-loop", &port,
+		 NULL);
 
-    count_beats_audio->sequencer_loop->port_value.ags_port_boolean = loop;
+    g_value_init(&value,
+		 G_TYPE_BOOLEAN);
+    g_value_set_boolean(&value,
+			loop);
 
+    ags_port_safe_write(port,
+			&value);
+
+    g_value_unset(&value);
+    
+    /* iterate */
     list = list->next;
   }
 
-  list = AGS_MACHINE(drum)->audio->recall;
+  g_list_free(start_list);
+
+  /* recall - count beats audio */
+  g_object_get(AGS_MACHINE(drum)->audio,
+	       "recall", &start_list,
+	       NULL);
+
+  list = start_list;
 
   while((list = ags_recall_find_type(list,
 				     AGS_TYPE_COUNT_BEATS_AUDIO)) != NULL){
+    GValue value = {0,};
+    
     count_beats_audio = AGS_COUNT_BEATS_AUDIO(list->data);
+    g_object_get(count_beats_audio,
+		 "sequencer-loop", &port,
+		 NULL);
 
-    count_beats_audio->sequencer_loop->port_value.ags_port_boolean = loop;
+    g_value_init(&value,
+		 G_TYPE_BOOLEAN);
+    g_value_set_boolean(&value,
+			loop);
 
+    ags_port_safe_write(port,
+			&value);
+
+    g_value_unset(&value);
+    
+    /* iterate */
     list = list->next;
   }
 
-  pthread_mutex_unlock(audio_mutex);
+  g_list_free(start_list);  
 }
 
 void
 ags_drum_length_spin_callback(GtkWidget *spin_button, AgsDrum *drum)
 {
   AgsWindow *window;
+  
+  AgsGuiThread *gui_thread;
 
   AgsApplySequencerLength *apply_sequencer_length;
-  
-  AgsMutexManager *mutex_manager;
-  AgsThread *main_loop;
-  AgsGuiThread *gui_thread;
 
   AgsApplicationContext *application_context;
   
   gdouble length;
 
-  pthread_mutex_t *application_mutex;
-
   /* get window and application_context  */
   window = (AgsWindow *) gtk_widget_get_toplevel((GtkWidget *) drum);
   
   application_context = (AgsApplicationContext *) window->application_context;
 
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
+  gui_thread = ags_ui_provider_get_gui_thread(AGS_UI_PROVIDER(application_context));
 
-  /* get audio loop */
-  pthread_mutex_lock(application_mutex);
-
-  main_loop = (AgsThread *) application_context->main_loop;
-
-  pthread_mutex_unlock(application_mutex);
-
-  /* get task thread */
-  gui_thread = (AgsGuiThread *) ags_thread_find_type(main_loop,
-						       AGS_TYPE_GUI_THREAD);
-
-  /*  */
+  /* task - apply length */
   length = GTK_SPIN_BUTTON(spin_button)->adjustment->value;
 
-  apply_sequencer_length = ags_apply_sequencer_length_new((GObject *) AGS_MACHINE(drum)->audio,
+  apply_sequencer_length = ags_apply_sequencer_length_new(AGS_MACHINE(drum)->audio,
 							  length);
 
   ags_gui_thread_schedule_task(gui_thread,
@@ -266,208 +242,208 @@ ags_drum_length_spin_callback(GtkWidget *spin_button, AgsDrum *drum)
 void
 ags_drum_index0_callback(GtkWidget *widget, AgsDrum *drum)
 {
-  AgsMutexManager *mutex_manager;
-
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  pthread_mutex_lock(application_mutex);
-  
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) AGS_MACHINE(drum)->audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
-  pthread_mutex_lock(audio_mutex);
-
   if(drum->selected0 != NULL){
     GtkToggleButton *toggle_button;
 
     if(GTK_TOGGLE_BUTTON(widget) != drum->selected0){
+      AgsPort *port;
+      
       AgsCopyPatternAudio *copy_pattern_audio;
-      GList *list;
+      
+      GList *start_list, *list;
+
       guint64 index0;
       
       toggle_button = drum->selected0;
       drum->selected0 = NULL;
-      gtk_toggle_button_set_active(toggle_button, FALSE);
-      drum->selected0 = (GtkToggleButton*) widget;
 
-      list = ags_recall_find_type(AGS_MACHINE(drum)->audio->play,
-				  AGS_TYPE_COPY_PATTERN_AUDIO);
+      gtk_toggle_button_set_active(toggle_button,
+				   FALSE);
+
+      drum->selected0 = (GtkToggleButton*) widget;
 
       /* calculate index 0 */
       AGS_MACHINE(drum)->bank_0 = 
 	index0 = ((guint) drum->selected0->button.label_text[0] - 'a');
 
-      if(list != NULL){
-	GValue value = {0,};
-
-	g_value_init(&value, G_TYPE_UINT64);
-	g_value_set_uint64(&value, index0);
-
-	copy_pattern_audio = AGS_COPY_PATTERN_AUDIO(list->data);
-	ags_port_safe_write(copy_pattern_audio->bank_index_0, &value);
-
-	g_value_unset(&value);
-      }
-
-      list = ags_recall_find_type(AGS_MACHINE(drum)->audio->recall,
+      /* play - set port */
+      g_object_get(AGS_MACHINE(drum)->audio,
+		   "play", &start_list,
+		   NULL);
+      
+      list = ags_recall_find_type(start_list,
 				  AGS_TYPE_COPY_PATTERN_AUDIO);
 
       if(list != NULL){
 	GValue value = {0,};
 
-	g_value_init(&value, G_TYPE_UINT64);
-	g_value_set_uint64(&value, index0);
+	g_value_init(&value,
+		     G_TYPE_UINT64);
+	
+	g_value_set_uint64(&value,
+			   index0);
 
 	copy_pattern_audio = AGS_COPY_PATTERN_AUDIO(list->data);
-	ags_port_safe_write(copy_pattern_audio->bank_index_0, &value);
+	g_object_get(copy_pattern_audio,
+		     "bank-index-0", &port,
+		     NULL);
+	
+	ags_port_safe_write(port,
+			    &value);
 
 	g_value_unset(&value);
       }
+
+      g_list_free(start_list);
+      
+      /* recall - set port */
+      g_object_get(AGS_MACHINE(drum)->audio,
+		   "recall", &start_list,
+		   NULL);
+      
+      list = ags_recall_find_type(start_list,
+				  AGS_TYPE_COPY_PATTERN_AUDIO);
+
+      if(list != NULL){
+	GValue value = {0,};
+
+	g_value_init(&value,
+		     G_TYPE_UINT64);
+	
+	g_value_set_uint64(&value,
+			   index0);
+
+	copy_pattern_audio = AGS_COPY_PATTERN_AUDIO(list->data);
+	g_object_get(copy_pattern_audio,
+		     "bank-index-0", &port,
+		     NULL);
+
+	ags_port_safe_write(port,
+			    &value);
+
+	g_value_unset(&value);
+      }
+      
+      g_list_free(start_list);
     }else if(! gtk_toggle_button_get_active(drum->selected0)){
       drum->selected0 = NULL;
-      gtk_toggle_button_set_active((GtkToggleButton *) widget, TRUE);
+
+      gtk_toggle_button_set_active((GtkToggleButton *) widget,
+				   TRUE);
+
       drum->selected0 = (GtkToggleButton*) widget;
     }
 
     ags_pattern_box_set_pattern(drum->pattern_box);
   }
-
-  pthread_mutex_unlock(audio_mutex);
 }
 
 void
 ags_drum_index1_callback(GtkWidget *widget, AgsDrum *drum)
 {
-  AgsMutexManager *mutex_manager;
-
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  pthread_mutex_lock(application_mutex);
- 
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) AGS_MACHINE(drum)->audio);
-  
-  pthread_mutex_unlock(application_mutex);
-
-  pthread_mutex_lock(audio_mutex);
-
   if(drum->selected1 != NULL){
     GtkToggleButton *toggle_button;
 
     if(GTK_TOGGLE_BUTTON(widget) != drum->selected1){
+      AgsPort *port;
+      
       AgsCopyPatternAudio *copy_pattern_audio;
-      GList *list;
+
+      GList *start_list, *list;
+
       guint64 index1;
       
       toggle_button = drum->selected1;
       drum->selected1 = NULL;
-      gtk_toggle_button_set_active(toggle_button, FALSE);
-      drum->selected1 = (GtkToggleButton*) widget;
 
-      list = ags_recall_find_type(AGS_MACHINE(drum)->audio->play,
-				  AGS_TYPE_COPY_PATTERN_AUDIO);
+      gtk_toggle_button_set_active(toggle_button,
+				   FALSE);
+
+      drum->selected1 = (GtkToggleButton*) widget;
 
       /* calculate index 1 */
       AGS_MACHINE(drum)->bank_1 = 
 	index1 = ((guint) g_ascii_strtoull(drum->selected1->button.label_text, NULL, 10)) - 1;
 
-      if(list != NULL){
-	GValue value = {0,};
-
-	g_value_init(&value, G_TYPE_UINT64);
-	g_value_set_uint64(&value, index1);
-
-	copy_pattern_audio = AGS_COPY_PATTERN_AUDIO(list->data);
-	ags_port_safe_write(copy_pattern_audio->bank_index_1, &value);
-
-	g_value_unset(&value);
-      }
-
-      list = ags_recall_find_type(AGS_MACHINE(drum)->audio->recall,
+      /* play - set port */
+      g_object_get(AGS_MACHINE(drum)->audio,
+		   "play", &start_list,
+		   NULL);
+      
+      list = ags_recall_find_type(start_list,
 				  AGS_TYPE_COPY_PATTERN_AUDIO);
 
       if(list != NULL){
 	GValue value = {0,};
 
-	g_value_init(&value, G_TYPE_UINT64);
-	g_value_set_uint64(&value, index1);
+	g_value_init(&value,
+		     G_TYPE_UINT64);
+	
+	g_value_set_uint64(&value,
+			   index1);
 
 	copy_pattern_audio = AGS_COPY_PATTERN_AUDIO(list->data);
-	ags_port_safe_write(copy_pattern_audio->bank_index_1, &value);
+	g_object_get(copy_pattern_audio,
+		     "bank-index-1", &port,
+		     NULL);
+
+	ags_port_safe_write(port,
+			    &value);
+
+	g_value_unset(&value);
+      }
+
+
+      g_list_free(start_list);
+      
+      /* recall - set port */
+      g_object_get(AGS_MACHINE(drum)->audio,
+		   "recall", &start_list,
+		   NULL);
+      
+      list = ags_recall_find_type(start_list,
+				  AGS_TYPE_COPY_PATTERN_AUDIO);
+
+      if(list != NULL){
+	GValue value = {0,};
+
+	g_value_init(&value,
+		     G_TYPE_UINT64);
+	g_value_set_uint64(&value,
+			   index1);
+
+	copy_pattern_audio = AGS_COPY_PATTERN_AUDIO(list->data);
+	g_object_get(copy_pattern_audio,
+		     "bank-index-1", &port,
+		     NULL);
+	
+	ags_port_safe_write(port,
+			    &value);
 
 	g_value_unset(&value);
       }
     }else if(!gtk_toggle_button_get_active(drum->selected1)){
       drum->selected1 = NULL;
-      gtk_toggle_button_set_active((GtkToggleButton *) widget, TRUE);
+
+      gtk_toggle_button_set_active((GtkToggleButton *) widget,
+				   TRUE);
+
       drum->selected1 = (GtkToggleButton*) widget;
     }
 
     ags_pattern_box_set_pattern(drum->pattern_box);
   }
-
-  pthread_mutex_unlock(audio_mutex);
 }
 
 void
-ags_drum_done_callback(AgsDrum *drum,
-		       AgsRecallID *recall_id,
+ags_drum_stop_callback(AgsDrum *drum,
+		       GList *recall_id, gint sound_scope,
 		       gpointer data)
 {
-  AgsAudio *audio;
-  
-  AgsMutexManager *mutex_manager;
-
-  GList *playback;
-
-  gboolean all_done;
-
-  pthread_mutex_t *application_mutex;
-  pthread_mutex_t *audio_mutex;
-
-  mutex_manager = ags_mutex_manager_get_instance();
-  application_mutex = ags_mutex_manager_get_application_mutex(mutex_manager);
-
-  audio = AGS_MACHINE(drum)->audio;
-
-  /* get audio mutex */
-  pthread_mutex_lock(application_mutex);
-
-  audio_mutex = ags_mutex_manager_lookup(mutex_manager,
-					 (GObject *) audio);
-  
-  pthread_mutex_unlock(application_mutex);
-  
-  /* check unset */
-  pthread_mutex_lock(audio_mutex);
-
-  playback = AGS_PLAYBACK_DOMAIN(audio->playback_domain)->playback;
-
-  all_done = TRUE;
-
-  while(playback != NULL){
-    if(AGS_PLAYBACK(playback->data)->recall_id[1] != NULL){
-      all_done = FALSE;
-      break;
-    }
-
-    playback = playback->next;
+  if(sound_scope != AGS_SOUND_SCOPE_SEQUENCER){
+    return;
   }
-
-  pthread_mutex_unlock(audio_mutex);
-
+  
   /* all done */
-  if(all_done){
-    ags_led_array_unset_all(drum->pattern_box->hled_array);
-  }
+  ags_led_array_unset_all(drum->pattern_box->hled_array);
 }
