@@ -1,5 +1,5 @@
 /* GSequencer - Advanced GTK Sequencer
- * Copyright (C) 2005-2015 Joël Krähemann
+ * Copyright (C) 2005-2018 Joël Krähemann
  *
  * This file is part of GSequencer.
  *
@@ -32,8 +32,6 @@
 #include <string.h>
 #include <pwd.h>
 
-#include <pthread.h>
-
 #include <ags/i18n.h>
 
 void ags_application_context_class_init(AgsApplicationContextClass *application_context);
@@ -47,11 +45,22 @@ void ags_application_context_get_property(GObject *gobject,
 					  guint prop_id,
 					  GValue *value,
 					  GParamSpec *param_spec);
-void ags_application_context_connect(AgsConnectable *connectable);
-void ags_application_context_disconnect(AgsConnectable *connectable);
 void ags_application_context_dispose(GObject *gobject);
 void ags_application_context_finalize(GObject *gobject);
 
+AgsUUID* ags_application_context_get_uuid(AgsConnectable *connectable);
+gboolean ags_application_context_has_resource(AgsConnectable *connectable);
+gboolean ags_application_context_is_ready(AgsConnectable *connectable);
+void ags_application_context_add_to_registry(AgsConnectable *connectable);
+void ags_application_context_remove_from_registry(AgsConnectable *connectable);
+xmlNode* ags_application_context_list_resource(AgsConnectable *connectable);
+xmlNode* ags_application_context_xml_compose(AgsConnectable *connectable);
+void ags_application_context_xml_parse(AgsConnectable *connectable,
+				       xmlNode *node);
+gboolean ags_application_context_is_connected(AgsConnectable *connectable);
+void ags_application_context_connect(AgsConnectable *connectable);
+void ags_application_context_disconnect(AgsConnectable *connectable);
+
 void ags_application_context_real_load_config(AgsApplicationContext *application_context);
 
 void ags_application_context_real_prepare(AgsApplicationContext *application_context);
@@ -84,8 +93,8 @@ enum{
 
 enum{
   PROP_0,
-  PROP_APPLICATION_MUTEX,
   PROP_MAIN_LOOP,
+  PROP_TASK_THREAD,
   PROP_CONFIG,
   PROP_FILE,
 };
@@ -95,22 +104,24 @@ static guint application_context_signals[LAST_SIGNAL];
 
 AgsApplicationContext *ags_application_context = NULL;
 
+static pthread_mutex_t ags_application_context_class_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 GType
 ags_application_context_get_type()
 {
   static volatile gsize g_define_type_id__volatile = 0;
 
   if(g_once_init_enter (&g_define_type_id__volatile)){
-    GType ags_type_application_context;
-    
+    GType ags_type_application_context = 0;
+
     static const GTypeInfo ags_application_context_info = {
-      sizeof(AgsApplicationContextClass),
+      sizeof (AgsApplicationContextClass),
       NULL, /* base_init */
       NULL, /* base_finalize */
       (GClassInitFunc) ags_application_context_class_init,
       NULL, /* class_finalize */
       NULL, /* class_data */
-      sizeof(AgsApplicationContext),
+      sizeof (AgsApplicationContext),
       0,    /* n_preallocs */
       (GInstanceInitFunc) ags_application_context_init,
     };
@@ -130,7 +141,7 @@ ags_application_context_get_type()
 				AGS_TYPE_CONNECTABLE,
 				&ags_connectable_interface_info);
 
-    g_once_init_leave (&g_define_type_id__volatile, ags_type_application_context);
+    g_once_init_leave(&g_define_type_id__volatile, ags_type_application_context);
   }
 
   return g_define_type_id__volatile;
@@ -140,6 +151,7 @@ void
 ags_application_context_class_init(AgsApplicationContextClass *application_context)
 {
   GObjectClass *gobject;
+
   GParamSpec *param_spec;
 
   ags_application_context_parent_class = g_type_class_peek_parent(application_context);
@@ -159,23 +171,39 @@ ags_application_context_class_init(AgsApplicationContextClass *application_conte
    *
    * The assigned main-loop.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("main-loop",
-				   i18n_pspec("main-loop of application context"),
-				   i18n_pspec("The main-loop what application context is running in"),
+				   i18n_pspec("main loop of application context"),
+				   i18n_pspec("The main loop of application context running"),
 				   G_TYPE_OBJECT,
 				   G_PARAM_READABLE | G_PARAM_WRITABLE);
   g_object_class_install_property(gobject,
 				  PROP_MAIN_LOOP,
 				  param_spec);
 
+  /**
+   * AgsApplicationContext:task-thread:
+   *
+   * The assigned task thread.
+   * 
+   * Since: 2.0.0
+   */
+  param_spec = g_param_spec_object("task-thread",
+				   i18n_pspec("task thread"),
+				   i18n_pspec("The task thread"),
+				   G_TYPE_OBJECT,
+				   G_PARAM_READABLE | G_PARAM_WRITABLE);
+  g_object_class_install_property(gobject,
+				  PROP_TASK_THREAD,
+				  param_spec);
+
   /**
    * AgsApplicationContext:config:
    *
    * The assigned config.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("config",
 				   i18n_pspec("config of application context"),
@@ -191,7 +219,7 @@ ags_application_context_class_init(AgsApplicationContextClass *application_conte
    *
    * The assigned file.
    * 
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   param_spec = g_param_spec_object("file",
 				   i18n_pspec("file of application context"),
@@ -219,7 +247,7 @@ ags_application_context_class_init(AgsApplicationContextClass *application_conte
    *
    * The ::load-config notifies to load configuration.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   application_context_signals[LOAD_CONFIG] =
     g_signal_new("load-config",
@@ -237,7 +265,7 @@ ags_application_context_class_init(AgsApplicationContextClass *application_conte
    * The ::prepare signal should be implemented to prepare
    * your application context.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   application_context_signals[PREPARE] =
     g_signal_new("prepare",
@@ -255,7 +283,7 @@ ags_application_context_class_init(AgsApplicationContextClass *application_conte
    * The ::setup signal should be implemented to setup
    * your application context.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   application_context_signals[SETUP] =
     g_signal_new("setup",
@@ -273,7 +301,7 @@ ags_application_context_class_init(AgsApplicationContextClass *application_conte
    * The ::register-types signal should be implemented to load
    * your types.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   application_context_signals[REGISTER_TYPES] =
     g_signal_new("register-types",
@@ -290,7 +318,7 @@ ags_application_context_class_init(AgsApplicationContextClass *application_conte
    *
    * The ::quit notifies to load configuration.
    *
-   * Since: 1.0.0
+   * Since: 2.0.0
    */
   application_context_signals[QUIT] =
     g_signal_new("quit",
@@ -305,8 +333,23 @@ ags_application_context_class_init(AgsApplicationContextClass *application_conte
 void
 ags_application_context_connectable_interface_init(AgsConnectableInterface *connectable)
 {
+  connectable->get_uuid = ags_application_context_get_uuid;
+  connectable->has_resource = ags_application_context_has_resource;
+
+  connectable->is_ready = ags_application_context_is_ready;
+  connectable->add_to_registry = ags_application_context_add_to_registry;
+  connectable->remove_from_registry = ags_application_context_remove_from_registry;
+
+  connectable->list_resource = ags_application_context_list_resource;
+  connectable->xml_compose = ags_application_context_xml_compose;
+  connectable->xml_parse = ags_application_context_xml_parse;
+
+  connectable->is_connected = ags_application_context_is_connected;
   connectable->connect = ags_application_context_connect;
   connectable->disconnect = ags_application_context_disconnect;
+
+  connectable->connect_connection = NULL;
+  connectable->disconnect_connection = NULL;
 }
 
 void
@@ -317,6 +360,25 @@ ags_application_context_init(AgsApplicationContext *application_context)
 
   application_context->flags = 0;
 
+  application_context->obj_mutexattr = (pthread_mutexattr_t *) malloc(sizeof(pthread_mutexattr_t));
+
+  pthread_mutexattr_init(application_context->obj_mutexattr);
+  pthread_mutexattr_settype(application_context->obj_mutexattr,
+			    PTHREAD_MUTEX_RECURSIVE);
+
+#ifdef __linux__
+  pthread_mutexattr_setprotocol(application_context->obj_mutexattr,
+				PTHREAD_PRIO_INHERIT);
+#endif
+
+  
+  application_context->obj_mutex = (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
+  pthread_mutex_init(application_context->obj_mutex, application_context->obj_mutexattr);
+
+  /* uuid */
+  application_context->uuid = ags_uuid_alloc();
+  ags_uuid_generate(application_context->uuid);
+  
   application_context->argc = 0;
   application_context->argv = NULL;
   
@@ -327,21 +389,6 @@ ags_application_context_init(AgsApplicationContext *application_context)
   application_context->domain = NULL;
   application_context->config = NULL;
 
-  application_context->mutexattr = (pthread_mutexattr_t *) malloc(sizeof(pthread_mutexattr_t));
-
-  pthread_mutexattr_init(application_context->mutexattr);
-  pthread_mutexattr_settype(application_context->mutexattr,
-			    PTHREAD_MUTEX_RECURSIVE);
-
-#ifdef __linux__
-  pthread_mutexattr_setprotocol(application_context->mutexattr,
-				PTHREAD_PRIO_INHERIT);
-#endif
-
-  
-  application_context->mutex = (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
-  pthread_mutex_init(application_context->mutex, application_context->mutexattr);
-
   application_context->main_loop = NULL;
   application_context->task_thread = NULL;
   application_context->autosave_thread = NULL;
@@ -359,37 +406,29 @@ ags_application_context_set_property(GObject *gobject,
 {
   AgsApplicationContext *application_context;
 
+  pthread_mutex_t *application_context_mutex;
+  
   application_context = AGS_APPLICATION_CONTEXT(gobject);
 
+  /* get application context mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = application_context->obj_mutex;
+  
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+  
   switch(prop_id){
-  case PROP_CONFIG:
-    {
-      AgsConfig *config;
-      
-      config = (AgsConfig *) g_value_get_object(value);
-
-      if(config == application_context->config){
-	return;
-      }
-
-      if(application_context->config != NULL){
-	g_object_unref(application_context->config);
-      }
-      
-      if(config != NULL){
-	g_object_ref(G_OBJECT(config));
-      }
-      
-      application_context->config = config;
-    }
-    break;
   case PROP_MAIN_LOOP:
     {
       GObject *main_loop;
       
       main_loop = (GObject *) g_value_get_object(value);
 
-      if(main_loop == application_context->main_loop){
+      pthread_mutex_lock(application_context_mutex);
+      
+      if(main_loop == application_context->main_loop){  
+	pthread_mutex_unlock(application_context_mutex);
+
 	return;
       }
 
@@ -402,6 +441,62 @@ ags_application_context_set_property(GObject *gobject,
       }
       
       application_context->main_loop = main_loop;
+  
+      pthread_mutex_unlock(application_context_mutex);
+    }
+    break;
+  case PROP_TASK_THREAD:
+    {
+      GObject *task_thread;
+      
+      task_thread = (GObject *) g_value_get_object(value);
+
+      pthread_mutex_lock(application_context_mutex);
+      
+      if(task_thread == application_context->task_thread){  
+	pthread_mutex_unlock(application_context_mutex);
+
+	return;
+      }
+
+      if(application_context->task_thread != NULL){
+	g_object_unref(application_context->task_thread);
+      }
+      
+      if(task_thread != NULL){
+	g_object_ref(G_OBJECT(task_thread));
+      }
+      
+      application_context->task_thread = task_thread;
+  
+      pthread_mutex_unlock(application_context_mutex);
+    }
+    break;
+  case PROP_CONFIG:
+    {
+      AgsConfig *config;
+      
+      config = (AgsConfig *) g_value_get_object(value);
+
+      pthread_mutex_lock(application_context_mutex);
+      
+      if(config == application_context->config){  
+	pthread_mutex_unlock(application_context_mutex);
+
+	return;
+      }
+
+      if(application_context->config != NULL){
+	g_object_unref(application_context->config);
+      }
+      
+      if(config != NULL){
+	g_object_ref(G_OBJECT(config));
+      }
+      
+      application_context->config = config;
+  
+      pthread_mutex_unlock(application_context_mutex);
     }
     break;
   case PROP_FILE:
@@ -410,7 +505,11 @@ ags_application_context_set_property(GObject *gobject,
       
       file = (AgsFile *) g_value_get_object(value);
 
+      pthread_mutex_lock(application_context_mutex);
+
       if(file == application_context->file){
+	pthread_mutex_unlock(application_context_mutex);
+	
 	return;
       }
 
@@ -423,6 +522,8 @@ ags_application_context_set_property(GObject *gobject,
       }
       
       application_context->file = (AgsFile *) file;
+  
+      pthread_mutex_unlock(application_context_mutex);
     }
     break;
   default:
@@ -440,22 +541,52 @@ ags_application_context_get_property(GObject *gobject,
 {
   AgsApplicationContext *application_context;
 
+  pthread_mutex_t *application_context_mutex;
+  
   application_context = AGS_APPLICATION_CONTEXT(gobject);
 
+  /* get application context mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = application_context->obj_mutex;
+  
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
   switch(prop_id){
-  case PROP_CONFIG:
-    {
-      g_value_set_object(value, application_context->config);
-    }
-    break;
   case PROP_MAIN_LOOP:
     {
+      pthread_mutex_lock(application_context_mutex);
+
       g_value_set_object(value, application_context->main_loop);
+  
+      pthread_mutex_unlock(application_context_mutex);
+    }
+    break;
+  case PROP_TASK_THREAD:
+    {
+      pthread_mutex_lock(application_context_mutex);
+
+      g_value_set_object(value, application_context->task_thread);
+  
+      pthread_mutex_unlock(application_context_mutex);
+    }
+    break;
+  case PROP_CONFIG:
+    {
+      pthread_mutex_lock(application_context_mutex);
+
+      g_value_set_object(value, application_context->config);
+  
+      pthread_mutex_unlock(application_context_mutex);
     }
     break;
   case PROP_FILE:
     {
+      pthread_mutex_lock(application_context_mutex);
+
       g_value_set_object(value, application_context->file);
+  
+      pthread_mutex_unlock(application_context_mutex);
     }
     break;
   default:
@@ -464,41 +595,6 @@ ags_application_context_get_property(GObject *gobject,
   }
 }
 
-void
-ags_application_context_connect(AgsConnectable *connectable)
-{
-  AgsApplicationContext *application_context;
-
-  application_context = AGS_APPLICATION_CONTEXT(connectable);
-
-  if((AGS_APPLICATION_CONTEXT_CONNECTED & (application_context->flags)) != 0)
-    return;
-
-  application_context->flags |= AGS_APPLICATION_CONTEXT_CONNECTED;
-
-  if((AGS_APPLICATION_CONTEXT_DEFAULT & (application_context->flags)) != 0){
-    GList *list;
-
-    list = application_context->sibling;
-
-    while(list != NULL){
-      if(application_context != list->data){
-	ags_connectable_connect(AGS_CONNECTABLE(list->data));
-      }
-
-      list = list->next;
-    }
-  }
-
-  /* note main loop won't connect here */
-}
-
-void
-ags_application_context_disconnect(AgsConnectable *connectable)
-{
-  /* empty */
-}
-
 void
 ags_application_context_dispose(GObject *gobject)
 {
@@ -576,11 +672,11 @@ ags_application_context_finalize(GObject *gobject)
   application_context = AGS_APPLICATION_CONTEXT(gobject);
 
   /* application mutex */
-  pthread_mutexattr_destroy(application_context->mutexattr);
-  free(application_context->mutexattr);
+  pthread_mutexattr_destroy(application_context->obj_mutexattr);
+  free(application_context->obj_mutexattr);
 
-  pthread_mutex_destroy(application_context->mutex);
-  free(application_context->mutex);
+  pthread_mutex_destroy(application_context->obj_mutex);
+  free(application_context->obj_mutex);
 
   /* log */
   if(application_context->log != NULL){
@@ -632,21 +728,310 @@ ags_application_context_finalize(GObject *gobject)
   G_OBJECT_CLASS(ags_application_context_parent_class)->finalize(gobject);
 }
 
+AgsUUID*
+ags_application_context_get_uuid(AgsConnectable *connectable)
+{
+  AgsApplicationContext *application_context;
+  
+  AgsUUID *ptr;
+
+  pthread_mutex_t *application_context_mutex;
+
+  application_context = AGS_APPLICATION_CONTEXT(connectable);
+
+  /* get application_context mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = application_context->obj_mutex;
+  
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* get UUID */
+  pthread_mutex_lock(application_context_mutex);
+
+  ptr = application_context->uuid;
+
+  pthread_mutex_unlock(application_context_mutex);
+  
+  return(ptr);
+}
+
+gboolean
+ags_application_context_has_resource(AgsConnectable *connectable)
+{
+  return(TRUE);
+}
+
+gboolean
+ags_application_context_is_ready(AgsConnectable *connectable)
+{
+  AgsApplicationContext *application_context;
+  
+  gboolean is_ready;
+
+  pthread_mutex_t *application_context_mutex;
+
+  application_context = AGS_APPLICATION_CONTEXT(connectable);
+
+  /* get application_context mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = application_context->obj_mutex;
+  
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* check is added */
+  pthread_mutex_lock(application_context_mutex);
+
+  is_ready = (((AGS_APPLICATION_CONTEXT_ADDED_TO_REGISTRY & (application_context->flags)) != 0) ? TRUE: FALSE);
+
+  pthread_mutex_unlock(application_context_mutex);
+  
+  return(is_ready);
+}
+
+void
+ags_application_context_add_to_registry(AgsConnectable *connectable)
+{
+  AgsApplicationContext *application_context;
+  
+  if(ags_connectable_is_ready(connectable)){
+    return;
+  }
+  
+  application_context = AGS_APPLICATION_CONTEXT(connectable);
+
+  ags_application_context_set_flags(application_context, AGS_APPLICATION_CONTEXT_ADDED_TO_REGISTRY);
+}
+
+void
+ags_application_context_remove_from_registry(AgsConnectable *connectable)
+{
+  AgsApplicationContext *application_context;
+
+  if(!ags_connectable_is_ready(connectable)){
+    return;
+  }
+
+  application_context = AGS_APPLICATION_CONTEXT(connectable);
+
+  ags_application_context_unset_flags(application_context, AGS_APPLICATION_CONTEXT_ADDED_TO_REGISTRY);
+}
+
+xmlNode*
+ags_application_context_list_resource(AgsConnectable *connectable)
+{
+  xmlNode *node;
+  
+  node = NULL;
+
+  //TODO:JK: implement me
+  
+  return(node);
+}
+
+xmlNode*
+ags_application_context_xml_compose(AgsConnectable *connectable)
+{
+  xmlNode *node;
+  
+  node = NULL;
+
+  //TODO:JK: implement me
+  
+  return(node);
+}
+
+void
+ags_application_context_xml_parse(AgsConnectable *connectable,
+				  xmlNode *node)
+{
+  //TODO:JK: implement me
+}
+
+gboolean
+ags_application_context_is_connected(AgsConnectable *connectable)
+{
+  AgsApplicationContext *application_context;
+  
+  gboolean is_connected;
+
+  pthread_mutex_t *application_context_mutex;
+
+  application_context = AGS_APPLICATION_CONTEXT(connectable);
+
+  /* get application_context mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = application_context->obj_mutex;
+  
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* check is connected */
+  pthread_mutex_lock(application_context_mutex);
+
+  is_connected = (((AGS_APPLICATION_CONTEXT_CONNECTED & (application_context->flags)) != 0) ? TRUE: FALSE);
+  
+  pthread_mutex_unlock(application_context_mutex);
+  
+  return(is_connected);
+}
+
+void
+ags_application_context_connect(AgsConnectable *connectable)
+{
+  AgsApplicationContext *application_context;
+
+  if(ags_connectable_is_connected(connectable)){
+    return;
+  }
+
+  application_context = AGS_APPLICATION_CONTEXT(connectable);
+
+  ags_application_context_set_flags(application_context, AGS_APPLICATION_CONTEXT_CONNECTED);
+}
+
+void
+ags_application_context_disconnect(AgsConnectable *connectable)
+{
+  AgsApplicationContext *application_context;
+
+  if(!ags_connectable_is_connected(connectable)){
+    return;
+  }
+
+  application_context = AGS_APPLICATION_CONTEXT(connectable);
+
+  ags_application_context_unset_flags(application_context, AGS_APPLICATION_CONTEXT_CONNECTED);
+}
+
+/**
+ * ags_application_context_get_class_mutex:
+ * 
+ * Use this function's returned mutex to access mutex fields.
+ *
+ * Returns: the class mutex
+ * 
+ * Since: 2.0.0
+ */
+pthread_mutex_t*
+ags_application_context_get_class_mutex()
+{
+  return(&ags_application_context_class_mutex);
+}
+
+/**
+ * ags_application_context_test_flags:
+ * @application_context: the #AgsApplicationContext
+ * @flags: the flags
+ *
+ * Test @flags to be set on @application_context.
+ * 
+ * Returns: %TRUE if flags are set, else %FALSE
+ *
+ * Since: 2.0.0
+ */
+gboolean
+ags_application_context_test_flags(AgsApplicationContext *application_context, guint flags)
+{
+  gboolean retval;  
+  
+  pthread_mutex_t *application_context_mutex;
+
+  if(!AGS_IS_APPLICATION_CONTEXT(application_context)){
+    return(FALSE);
+  }
+
+  /* get application_context mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = application_context->obj_mutex;
+  
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* test */
+  pthread_mutex_lock(application_context_mutex);
+
+  retval = (flags & (application_context->flags)) ? TRUE: FALSE;
+  
+  pthread_mutex_unlock(application_context_mutex);
+
+  return(retval);
+}
+
+/**
+ * ags_application_context_set_flags:
+ * @application_context: the #AgsApplicationContext
+ * @flags: see enum AgsApplicationContextFlags
+ *
+ * Enable a feature of #AgsApplicationContext.
+ *
+ * Since: 2.0.0
+ */
+void
+ags_application_context_set_flags(AgsApplicationContext *application_context, guint flags)
+{
+  guint application_context_flags;
+  
+  pthread_mutex_t *application_context_mutex;
+
+  if(!AGS_IS_APPLICATION_CONTEXT(application_context)){
+    return;
+  }
+
+  /* get application_context mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = application_context->obj_mutex;
+  
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* set flags */
+  pthread_mutex_lock(application_context_mutex);
+
+  application_context->flags |= flags;
+  
+  pthread_mutex_unlock(application_context_mutex);
+}
+    
+/**
+ * ags_application_context_unset_flags:
+ * @application_context: the #AgsApplicationContext
+ * @flags: see enum AgsApplicationContextFlags
+ *
+ * Disable a feature of AgsApplicationContext.
+ *
+ * Since: 2.0.0
+ */
+void
+ags_application_context_unset_flags(AgsApplicationContext *application_context, guint flags)
+{
+  guint application_context_flags;
+  
+  pthread_mutex_t *application_context_mutex;
+
+  if(!AGS_IS_APPLICATION_CONTEXT(application_context)){
+    return;
+  }
+
+  /* get application_context mutex */
+  pthread_mutex_lock(ags_application_context_get_class_mutex());
+  
+  application_context_mutex = application_context->obj_mutex;
+  
+  pthread_mutex_unlock(ags_application_context_get_class_mutex());
+
+  /* unset flags */
+  pthread_mutex_lock(application_context_mutex);
+
+  application_context->flags &= (~flags);
+  
+  pthread_mutex_unlock(application_context_mutex);
+}
+
 void
 ags_application_context_real_load_config(AgsApplicationContext *application_context)
 {
-  AgsConfig *config;
-
-  if(application_context == NULL){
-    return;
-  }
-
-  config = application_context->config;
-
-  if(config == NULL){
-    return;
-  }
-
   //TODO:JK: implement me
 }
 
@@ -656,7 +1041,7 @@ ags_application_context_real_load_config(AgsApplicationContext *application_cont
  *
  * Signal to load and parse configuration.
  *
- * Since 0.7.0
+ * Since: 2.0.0
  */
 void
 ags_application_context_load_config(AgsApplicationContext *application_context)
@@ -676,6 +1061,14 @@ ags_application_context_real_prepare(AgsApplicationContext *application_context)
   ags_log_add_message(ags_log_get_instance(), "prepare Advanced Gtk+ Sequencer");
 }
 
+/**
+ * ags_application_context_prepare:
+ * @application_context: the #AgsApplicationContext
+ *
+ * Prepare @application_context.
+ *
+ * Since: 2.0.0
+ */
 void
 ags_application_context_prepare(AgsApplicationContext *application_context)
 {
@@ -694,6 +1087,14 @@ ags_application_context_real_setup(AgsApplicationContext *application_context)
   ags_log_add_message(ags_log_get_instance(), "setup Advanced Gtk+ Sequencer");
 }
 
+/**
+ * ags_application_context_setup:
+ * @application_context: the #AgsApplicationContext
+ *
+ * Setup @application_context.
+ *
+ * Since: 2.0.0
+ */
 void
 ags_application_context_setup(AgsApplicationContext *application_context)
 {
@@ -718,7 +1119,7 @@ ags_application_context_real_register_types(AgsApplicationContext *application_c
  *
  * Notification to register your types.
  *
- * Since 0.7.0
+ * Since: 2.0.0
  */
 void
 ags_application_context_register_types(AgsApplicationContext *application_context)
@@ -732,66 +1133,6 @@ ags_application_context_register_types(AgsApplicationContext *application_contex
   g_object_unref(application_context);
 }
 
-void
-ags_application_context_add_sibling(AgsApplicationContext *application_context,
-				    AgsApplicationContext *sibling)
-{
-  //TODO:JK: implement me
-}
-
-void
-ags_application_context_remove_sibling(AgsApplicationContext *application_context,
-				       AgsApplicationContext *sibling)
-{
-  //TODO:JK: implement me
-}
-
-/**
- * ags_application_context_find_default:
- * @application_context: the #GList-struct containing #AgsApplicationContext
- *
- * Find default context in @application_context #GList-struct containing
- * #AgsApplicationContext.
- *
- * Since: 1.0.0
- */
-AgsApplicationContext*
-ags_application_context_find_default(GList *application_context)
-{
-  while(application_context != NULL){
-    if((AGS_APPLICATION_CONTEXT_DEFAULT & (AGS_APPLICATION_CONTEXT(application_context->data)->flags)) != 0){
-      return(application_context->data);
-    }
-    
-    application_context = application_context->next;
-  }
-
-  return(NULL);
-}
-
-/**
- * ags_application_context_find_main_loop:
- * @application_context: the #GList-struct containing #AgsApplicationContext
- *
- * Find :main-loop in @application_context #GList-struct containing
- * #AgsApplicationContext.
- *
- * Since: 1.0.0
- */
-GList*
-ags_application_context_find_main_loop(GList *application_context)
-{
-  while(application_context != NULL){
-    if(AGS_APPLICATION_CONTEXT(application_context->data)->main_loop != NULL){
-      break;
-    }
-    
-    application_context = application_context->next;
-  }
-  
-  return(application_context);
-}
-
 void
 ags_application_context_real_quit(AgsApplicationContext *application_context)
 {
@@ -801,11 +1142,11 @@ ags_application_context_real_quit(AgsApplicationContext *application_context)
 
 /**
  * ags_application_context_quit:
- * @application_context: the context to quit
+ * @application_context: the #AgsApplicationContext
  *
  * Calls exit()
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 void
 ags_application_context_quit(AgsApplicationContext *application_context)
@@ -825,16 +1166,22 @@ ags_application_context_quit(AgsApplicationContext *application_context)
  *
  * Returns: the #AgsApplicationContext instance
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsApplicationContext*
 ags_application_context_get_instance()
 {
+  static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
+
+  pthread_mutex_lock(&mutex);
+  
   if(ags_application_context == NULL){
     ags_application_context = ags_application_context_new(NULL,
 							  NULL);
   }
 
+  pthread_mutex_unlock(&mutex);
+  
   return(ags_application_context);
 }
 
@@ -847,7 +1194,7 @@ ags_application_context_get_instance()
  * 
  * Returns: the #AgsApplicationContext instance
  *
- * Since: 1.0.0
+ * Since: 2.0.0
  */
 AgsApplicationContext*
 ags_application_context_new(GObject *main_loop,
